unit utTraceDB;

interface

uses windows, classes, db, dbtables, sysutils, bde;

type
  TOldQuery = TQuery;
  TOldStoredProc = TStoredProc;

  TNewQuery = class( TQuery )
  private
    function GetDField(const Name: String): TDateTime;
    function GetFField(const Name: String): Double;
    function GetIField(const Name: String): Integer;
    function GetSField(const Name: String): String;
    function GetDParam(const Name: String): TDateTime;
    function GetFParam(const Name: String): Double;
    function GetIParam(const Name: String): Integer;
    function GetSParam(const Name: String): String;
    procedure SetDParam(const Name: String; const Value: TDateTime);
    procedure SetFParam(const Name: String; const Value: Double);
    procedure SetIParam(const Name: String; const Value: Integer);
    procedure SetSParam(const Name, Value: String);
    function GetDFieldN(Idx: Integer): TDateTime;
    function GetFFieldN(Idx: Integer): Double;
    function GetIFieldN(Idx: Integer): Integer;
    function GetSFieldN(Idx: Integer): String;
    function GetNullField(const Name: String): Boolean;
    function GetBField(const Name: String): TStream;
    function GetBFieldN(Idx: Integer): TStream;
    procedure SetBParam(const Name: String; const Value: TStream);
  protected
    procedure DoBeforeOpen; override;
    procedure DoAfterOpen; override;
  public
    CanCancel: Boolean;
    constructor Create( AOwner: TComponent ); override;
    procedure ExecSQL;
    procedure Cancel; override;
    property IField[ const Name: String ]: Integer read GetIField;
    property SField[ const Name: String ]: String read GetSField;
    property FField[ const Name: String ]: Double read GetFField;
    property DField[ const Name: String ]: TDateTime read GetDField;
    property BField[ const Name: String ]: TStream read GetBField;
    property IFieldN[ Idx: Integer ]: Integer read GetIFieldN;
    property SFieldN[ Idx: Integer ]: String read GetSFieldN;
    property FFieldN[ Idx: Integer ]: Double read GetFFieldN;
    property DFieldN[ Idx: Integer ]: TDateTime read GetDFieldN;
    property BFieldN[ Idx: Integer ]: TStream read GetBFieldN;
    property IParam[ const Name: String ]: Integer read GetIParam write SetIParam;
    property SParam[ const Name: String ]: String read GetSParam write SetSParam;
    property FParam[ const Name: String ]: Double read GetFParam write SetFParam;
    property DParam[ const Name: String ]: TDateTime read GetDParam write SetDParam;
    property BParam[ const Name: String ]: TStream write SetBParam;
    property NullField[ const Name: String ]: Boolean read GetNullField;
    procedure NullParam( const Name: String; FieldType: TFieldType = ftUnknown );
  end;

  TNewStoredProc = class( TStoredProc )
  private
  protected
    procedure DoBeforeOpen; override;
  public
    CanCancel: Boolean;
    procedure ExecProc;
    procedure Cancel; override;
  end;

type
  TQuery = TNewQuery;
  TStoredProc = TNewStoredProc;

var ExecutingQuery: TNewQuery;

implementation

uses utTraceLog;

{ TFakeQuery }

procedure TNewQuery.Cancel;
begin
  if not CanCancel then Exit;
  inherited;
end;

constructor TNewQuery.Create( AOwner: TComponent );
begin
  inherited;
  {if DefaultDatabase <> nil then
    Self.Database := DefaultDatabase;}
end;

procedure TNewQuery.DoAfterOpen;
begin
  if ExecutingQuery = Self then
    ExecutingQuery := nil;
  inherited;
end;

procedure TNewQuery.DoBeforeOpen;
begin
  ExecutingQuery := Self;
  inherited;
  AddToTraceLog( Name + '.Open: ' + SQL.Text, Params );
end;

procedure TNewQuery.ExecSQL;
begin
  ExecutingQuery := Self;
  TRY
    AddToTraceLog( Name + '.ExecSQL: ' + SQL.Text, Params );
    inherited ExecSQL;
  FINALLY
    ExecutingQuery := nil;
  END;
end;

{ TNewStoredProc }

procedure TNewStoredProc.Cancel;
begin
  if not CanCancel then Exit;
  inherited Cancel;
end;

procedure TNewStoredProc.DoBeforeOpen;
begin
  inherited;
  AddToTraceLog( Name + '.Open: ' + StoredProcName, Params );
end;

procedure TNewStoredProc.ExecProc;
begin
  AddToTraceLog( Name + '.ExecProc: ' + StoredProcName, Params );
  inherited ExecProc;
end;

function TNewQuery.GetBField(const Name: String): TStream;
begin
  Result := TBlobStream.Create( FieldByName( Name ) as TBlobField, bmRead );
end;

function TNewQuery.GetBFieldN(Idx: Integer): TStream;
begin
  Result := TBlobStream.Create( Fields[ Idx ] as TBlobField, bmRead );
end;

function TNewQuery.GetDField(const Name: String): TDateTime;
begin
  Result := FieldByName( Name ).AsDateTime;
end;

function TNewQuery.GetDFieldN(Idx: Integer): TDateTime;
begin
  Result := Fields[ Idx ].AsDateTime;
end;

function TNewQuery.GetDParam(const Name: String): TDateTime;
begin
  Result := ParamByName( Name ).AsDateTime;
end;

function TNewQuery.GetFField(const Name: String): Double;
begin
  Result := FieldByName( Name ).AsFloat;
end;

function TNewQuery.GetFFieldN(Idx: Integer): Double;
begin
  Result := Fields[ Idx ].AsFloat;
end;

function TNewQuery.GetFParam(const Name: String): Double;
begin
  Result := ParamByName( Name ).AsFloat;
end;

function TNewQuery.GetIField(const Name: String): Integer;
begin
  Result := FieldByName( Name ).AsInteger;
end;

function TNewQuery.GetIFieldN(Idx: Integer): Integer;
begin
  Result := Fields[ Idx ].AsInteger;
end;

function TNewQuery.GetIParam(const Name: String): Integer;
begin
  Result := ParamByName( Name ).AsInteger;
end;

function TNewQuery.GetNullField(const Name: String): Boolean;
begin
  Result := FieldByName( Name ).IsNull;
end;

function TNewQuery.GetSField(const Name: String): String;
begin
  Result := FieldByName( Name ).AsString;
end;

function TNewQuery.GetSFieldN(Idx: Integer): String;
begin
  Result := Fields[ Idx ].AsString;
end;

function TNewQuery.GetSParam(const Name: String): String;
begin
  Result := ParamByName( Name ).AsString;
end;

procedure TNewQuery.NullParam(const Name: String; FieldType: TFieldType);
begin
  ParamByName( Name ).Clear;
  if FieldType <> ftUnknown then
    ParamByName( Name ).DataType := FieldType;
end;

procedure TNewQuery.SetBParam(const Name: String; const Value: TStream);
var MS: TMemoryStream;
begin
  MS := TMemoryStream.Create;
  TRY
    MS.CopyFrom( Value, Value.Size - Value.Position );
    ParamByName( Name ).SetBlobData( MS.Memory, MS.Size );
  FINALLY
    MS.Free;
  END;
end;

procedure TNewQuery.SetDParam(const Name: String; const Value: TDateTime);
begin
  ParamByName( Name ).AsDateTime := Value;
end;

procedure TNewQuery.SetFParam(const Name: String; const Value: Double);
begin
  ParamByName( Name ).AsFloat := Value;
end;

procedure TNewQuery.SetIParam(const Name: String; const Value: Integer);
begin
  ParamByName( Name ).AsInteger := Value;
end;

procedure TNewQuery.SetSParam(const Name, Value: String);
begin
  ParamByName( Name ).AsString := Value;
end;

end.
